#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gnome
import gnome.ui
import gtk
import gtk.glade
import os
import snapshooter
from config import *
import re
from NewSnapshotWindow import NewSnapshotWindow

gnome.program_init ('Snapshooter', '0.01')


class ProtoSystemModel(gtk.ListStore):
    def __init__ (self):
        gtk.ListStore.__init__ (self, str)
        files = snapshooter.list_protosystems ()
        for file in files:
            row = self.append ()
            self[row][0] = file
        self.set_sort_column_id (0, gtk.SORT_ASCENDING)

class SnapshotsModel(gtk.ListStore):
    def __init__ (self, protosystem):
        gtk.ListStore.__init__ (self, str)
        files = snapshooter.list_snapshots (protosystem)
        for file in files:
            row = self.append ()
            self[row][0] = file
        self.set_sort_column_id (0, gtk.SORT_ASCENDING)

class MainWindow:
    def __init__ (self):
        glade_file = 'snapshooter.glade'
        if not os.access (glade_file, os.F_OK):
            glade_file = GLADEDIR + '/' + glade_file
        self.xml = gtk.glade.XML (glade_file)
        window = self.xml.get_widget ('main_window')
        window.connect ('destroy', gtk.main_quit)

        self.xml.get_widget ('new_snapshot_menu').connect ('activate', self.new_snapshot)
        tree_view = self.xml.get_widget ('protosystem_tree_view')
        c = gtk.TreeViewColumn (None, gtk.CellRendererText(), text=0)
        tree_view.get_selection().set_mode (gtk.SELECTION_BROWSE)
        tree_view.get_selection().connect ('changed', self.protosystem_selection_changed)
        tree_view.append_column (c)
        m = ProtoSystemModel ()
        tree_view.set_model (m)

        tree_view = self.xml.get_widget ('snapshots_tree_view')
        c = gtk.TreeViewColumn (None, gtk.CellRendererText(), text=0)
        tree_view.append_column (c)
        window.show ()

    def new_snapshot (self, menu):
        tree_view = self.xml.get_widget ('protosystem_tree_view')
        (model, row) = tree_view.get_selection().get_selected ()
        if row:
            protosystem = model[row][0]
            new_window = NewSnapshotWindow (protosystem, self.xml.get_widget ('main_window'))
            new_window.run ()
            self.xml.get_widget ('snapshots_tree_view').set_model (SnapshotsModel (protosystem))
            
    def protosystem_selection_changed (self, selection):
        (model, row) = selection.get_selected()
        snapshots_tree_view = self.xml.get_widget ('snapshots_tree_view')

        if row == None:
            snapshots_tree_view.set_model (None)
        else:
            protosystem = model[row][0]
            snapshots_tree_view.set_model (SnapshotsModel (protosystem))

if __name__ == '__main__':
    MainWindow ()

    gtk.main ()

