#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gtk
import gtk.glade
import os
import snapshooter
from config import *


class NewSnapshotWindow:

    def cancel (self):
        self.snapshooter.get_transfer ().cancel_transfer ()
        self.xml.get_widget ('new_snapshot_window').destroy ()
        gtk.main_quit ()

    def finish (self):
        self.xml.get_widget ('new_snapshot_window').destroy ()
        gtk.main_quit ()

    def run (self):
        gtk.main ()

    def update_progress (self, snapshooter, percent):
        self.xml.get_widget ('new_snapshot_progress').set_fraction (float (percent) / 100)

    def snapshot_finished (self):
        druid = self.xml.get_widget ('new_snapshot_druid')
        druid.set_show_finish (True)
        druid.set_buttons_sensitive (False, True, False, False)

    def prepare_initial (self, druid_page, druid):
        druid.set_buttons_sensitive (False, True, True, False)
        text = """Creating a new snapshot of the prototype system '%s'""" % self.protosystem
        self.xml.get_widget ('new_snapshot_initial_label').set_text (text)
        druid_page.emit_stop_by_name ('prepare')

    def prepare_update (self, druid_page, druid):
        druid.set_buttons_sensitive (False, False, False, False)
        self.snapshooter.start ()
        druid_page.emit_stop_by_name ('prepare')

    def __init__ (self, protosystem, parent=None):
        glade_file = 'NewSnapshotWindow.glade'
        if not os.access (glade_file, os.F_OK):
            glade_file = GLADEDIR + '/' + glade_file
        self.xml = gtk.glade.XML (glade_file, 'new_snapshot_window')
        self.protosystem = protosystem
        self.snapshooter = snapshooter.Snapshooter (self.protosystem,
                                                    snapshooter.get_next_id(self.protosystem))
        self.snapshooter.get_transfer ().connect ('update', self.update_progress)
        self.snapshooter.connect ('snapshot-finished', self.snapshot_finished)

        window = self.xml.get_widget ('new_snapshot_window')
        if parent: window.set_transient_for (parent)
        self.xml.get_widget ('new_snapshot_initial_druid_page').connect ('prepare', self.prepare_initial)
        self.xml.get_widget ('new_snapshot_update_druid_page').connect ('prepare', self.prepare_update)
        self.xml.get_widget ('new_snapshot_druid').set_page (self.xml.get_widget ('new_snapshot_initial_druid_page'))
        window.show ()

if __name__ == '__main__':
    NewSnapshotWindow ('urbox')
    gtk.main ()

