#!/usr/bin/python

#
#  lvm.py: Utilities relating to Logical Volume Manager
#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import fileinput
import os.path
from rhpl.executil import execWithCapture

class MountPoint:
    def __init__(self, device, mountpoint):
        self.device = device
        self.mountpoint = mountpoint

    def __str__(self):
        return "MountPoint(device:\"%s\", mountpoint:\"%s\")"%(self.device, self.mountpoint)

def get_all_mounted_volumes():
    result = []
    for line in fileinput.input("/proc/mounts"):
        split_line = line.split()
        result.append(MountPoint(split_line[0], split_line[1]))
    return result                
    

class VolumeGroup:
    def __init__(self, name):
        self.name = name;

    def get_device(self):
        return "/dev/"+self.name

    def __str__(self):
        return "VolumeGroup(\"%s\")"%self.name

    #def get_pvs(self):

    #def get_lvs(self):

class LogicalVolume:
    def __init__(self, lv_name, vg_name):
        self.lv_name = lv_name
        self.vg_name = vg_name

    def __str__(self):
        return "LogicalVolume(\"%s\",\"%s\")"%(self.lv_name, self.vg_name)    

# get all Logical Volumes on this computer as a list of LogicalVolume objects
def get_all_lvs():
    def parse_lv_display_line(line):
        fields = line.split(":")
        if len(fields)==13:
            lv = LogicalVolume(fields[0].lstrip(), fields[1].lstrip())
            return lv
        else:
            return None

    result = []
    
    str_result = execWithCapture("/usr/sbin/lvdisplay", ["lvdisplay", "-c"], False)
    
    lines = str_result.splitlines()
    for line in lines:
        lv = parse_lv_display_line(line)
        if lv!=None:
            result.append(lv)

    return result

# Get the device mounted at the given path, or None
def get_device_mounted_at (path):
    mount_points = get_all_mounted_volumes()

    for mp in mount_points:
        if os.path.normpath(mp.mountpoint)==os.path.normpath(path):
            return mp.device

    # Not found:
    return None

# is this path an LVM logical volume?:
def is_lvm_logical_volume(path):
    #print "is_lvm_logical_volume(%s)"%path
    device = get_device_mounted_at(path)
    if device!=None:
        lvs = get_all_lvs()
        for lv in lvs:
            if lv.lv_name == device:
                return True
            
    return False

def get_lv_mounted_at(path):
    device = get_device_mounted_at(path)
    if device!=None:
        lvs = get_all_lvs()
        for lv in lvs:
            if lv.lv_name == device:
                return lv
    return None
    

def get_device_mapper_targets():
    result = []
    str_result = execWithCapture("/sbin/dmsetup", ["dmsetup", "targets"], False)
    for line in str_result.splitlines():
        result.append(line.split(" ")[0])
    return result
    
def kernel_supports_snapshots():
    dmtargets = get_device_mapper_targets()
    return dmtargets.count("snapshot")>0 and dmtargets.count("snapshot-origin")>0 

def test():
    print "get_all_lvs(): %s"%get_all_lvs()
    print "get_all_mounted_volumes(): %s"%get_all_mounted_volumes()
    print "get_device_mounted_at(\"/proc/\"): %s"%get_device_mounted_at("/proc/")
    print "get_device_mounted_at(\"foo\"): %s"%get_device_mounted_at("foo")
    print "get_device_mounted_at(\"/srv/stateless/protosystems/DemoSystem\"): %s"%get_device_mounted_at("/srv/stateless/protosystems/DemoSystem")
    print "is_lvm_logical_volume(\"/proc\"): %s"%is_lvm_logical_volume("/proc")
    print "is_lvm_logical_volume(\"/srv/stateless/protosystems/DemoSystem\"): %s"%is_lvm_logical_volume("/srv/stateless/protosystems/DemoSystem")
    print "get_device_mapper_targets(): %s"%get_device_mapper_targets()
    print "kernel_supports_snapshots(): %s"%kernel_supports_snapshots()
            
                                 
        
    

