#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import ldap
import LDAPUtils
from config import StatelessConfig

def Usage ():
    print "stateless-clients: - Stateless Linux clients tool\n\nUsage: stateless-clients [-l|-a|-d|-m] [-c configuration] [-s snapshot] <mac-address-list>"

if __name__ == '__main__':
    import sys
    import getopt

    if len (sys.argv) == 1:
        Usage ()
        sys.exit (1)

    do_list = False
    do_add = False
    do_delete = False
    do_modify = False
    config_name = None
    snapshot_name = None
    mac_addresses = None

    class BadUsage: pass
    try:
        opts, args = getopt.getopt (sys.argv[1:],
                                    "ladmc:s:h",
                                    ["list", "add", "delete", "modify", "configuration", "snapshot", "help"])
        for opt, val in opts:
            if opt == '-l' or opt == '--list':
                do_list = True
            
            if opt == '-a' or opt == '--add':
                do_add = True
            
            if opt == '-d' or opt == '--delete':
                do_delete = True
            
            if opt == '-m' or opt == '--modify':
                do_modify = True

            if opt == '-l' or opt == '--list':
                do_list = True

            if opt == '-c' or opt == '--configuration':
                config_name = val
            
            if opt == '-s' or opt == '--snapshot':
                snapshot_name = val

            if opt == '-h' or opt == '--help':
                Usage ()
                sys.exit (0)

    except (getopt.error, BadUsage):
        Usage ()
        sys.exit (1)

    if len (args) > 0:
        mac_addresses = args

    if ((do_list and do_add) or (do_list and do_delete) or (do_list and do_modify) or
        (do_add and do_delete) or (do_add and do_modify) or (do_delete and do_modify)):
        Usage ()
        sys.exit (1)
        
    if (do_add or do_modify or do_delete) and mac_addresses == None:
        print "No MAC addresses specified"
        Usage ()
        sys.exit (0)

    if do_add and config_name == None:
        print "No configuration name specified"
        Usage ()
        sys.exit (0)

    cfg = StatelessConfig ()
    directory = LDAPUtils.ReadWriteDirectory (cfg)
    srv = LDAPUtils.StatelessService (directory)

    if do_list:
        clients = srv.get_client_list ()
        if clients != None:
            print "Clients:"
            for client in clients:
                mac_address   = client.get_attr_value ("macAddress")
                config_name   = client.get_attr_value ("statelessConfigurationName")
                snapshot_name = client.get_attr_value ("statelessSnapshotName")
                
                print "   %s" % (mac_address)

                if config_name != None:
                    print "      Configuration: %s" % (config_name)

                if snapshot_name != None:
                    print "      Snapshot: %s" % (snapshot_name)
        else:
            print "No clients"
        
        sys.exit (0)

    if do_add:
        for mac_address in mac_addresses:
            print "Adding %s" % (mac_address)

            assert config_name != None
            print "\tNew client's configuration is '%s'" % (config_name)
            
            if snapshot_name != None:
                print "\tNew client's snapshot is '%s'" % (snapshot_name)

            try:
                srv.add_client (mac_address, config_name, snapshot_name)
            except ldap.ALREADY_EXISTS:
                print "Client %s already exists, modifying" % (mac_address)
                srv.update_client (mac_address, config_name, snapshot_name)

            try:
                srv.add_client_status (mac_address, None, None, None);
            except ldap.ALREADY_EXISTS:
                pass

        sys.exit (0)

    if do_delete:
        for mac_address in mac_addresses:
            print "Deleting %s" % (mac_address)
            srv.delete_client (mac_address)

        sys.exit (0)
        
    if do_modify:
        for mac_address in mac_addresses:
            print "Modifying %s" % (mac_address)
            
            if config_name != None:
                print "\tClient's configuration is now %s" % (config_name)
            
            if snapshot_name != None:
                print "\tClient's snapshot is now %s" % (snapshot_name)

            srv.update_client (mac_address, config_name, snapshot_name)

        sys.exit (0)
