#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import ldap
import LDAPUtils
from config import StatelessConfig

def Usage ():
    print "stateless-servers: - Stateless Linux servers tool\n\nUsage: stateless-servers [-l|-a|-d|-m] [-n nfs-path] [-r rsync-path] <server-list>"

if __name__ == '__main__':
    import sys
    import getopt

    if len (sys.argv) == 1:
        Usage ()
        sys.exit (1)

    do_list = False
    do_add = False
    do_delete = False
    do_modify = False
    nfs_path = None
    rsync_path = None
    server_names = None

    class BadUsage: pass
    try:
        opts, args = getopt.getopt (sys.argv[1:],
                                    "ladmn:r:h",
                                    ["list", "add", "delete", "modify", "nfs-path", "rsync-path", "help"])
        for opt, val in opts:
            if opt == '-l' or opt == '--list':
                do_list = True
            
            if opt == '-a' or opt == '--add':
                do_add = True
            
            if opt == '-d' or opt == '--delete':
                do_delete = True
            
            if opt == '-m' or opt == '--modify':
                do_modify = True

            if opt == '-l' or opt == '--list':
                do_list = True

            if opt == '-n' or opt == '--nfs-path':
                nfs_path = val
            
            if opt == '-r' or opt == '--rsync-path':
                rsync_path = val

            if opt == '-h' or opt == '--help':
                Usage ()
                sys.exit (0)

    except (getopt.error, BadUsage):
        Usage ()
        sys.exit (1)

    if len (args) > 0:
        server_names = args

    if ((do_list and do_add) or (do_list and do_delete) or (do_list and do_modify) or
        (do_add and do_delete) or (do_add and do_modify) or (do_delete and do_modify)):
        Usage ()
        sys.exit (1)
        
    if (do_add or do_modify or do_delete) and server_names == None:
        print "No servers specified"
        Usage ()
        sys.exit (0)

    cfg = StatelessConfig ()
    directory = LDAPUtils.ReadWriteDirectory (cfg)
    srv = LDAPUtils.StatelessService (directory)

    if do_list:
        servers = srv.get_server_list ()
        if len (servers) > 0:
            print "Servers:"
            for server in servers:
                server_name = server.get_attr_value ("cn")
                nfs_path    = server.get_attr_value ("statelessNfsPath")
                rsync_path  = server.get_attr_value ("statelessRsyncPath")
                
                print "   %s" % (server.get_attr_value ("cn"))
                
                if nfs_path != None:
                    print "      NFS path: %s" % (nfs_path)

                if rsync_path != None:
                    print "      Rsync path: %s" % (rsync_path)
        else:
            print "No servers"
        
        sys.exit (0)

    if do_add:
        for server_name in server_names:
            print "Adding %s" % (server_name)

            if nfs_path != None:
                print "   New server's NFS path is %s" % (nfs_path)
            
            if rsync_path != None:
                print "   New server's rsync path is %s" % (rsync_path)

            try:
                srv.add_server (server_name, nfs_path, rsync_path)
            except ldap.ALREADY_EXISTS:
                print "Server %s already exists, modifying" % (server_name)
                srv.update_server (server_name, nfs_path, rsync_path)

        sys.exit (0)

    if do_delete:
        for server_name in server_names:
            print "Deleting %s" % (server_name)
            srv.delete_server (server_name)

        sys.exit (0)
        
    if do_modify:
        for server_name in server_names:
            print "Modifying %s" % (server_name)
            
            if nfs_path != None:
                print "   Server's NFS path is now %s" % (nfs_path)
            
            if nfs_path != None:
                print "   Server's rsync path is now %s" % (rsync_path)

            srv.update_server (server_name, nfs_path, rsync_path)

        sys.exit (0)
