#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import gtk
import snapshooter

# Taken from up2date
class progressBar:
    def __init__ (self):
        self.last_percent = -1
        self.hashes_printed = -1

    def update (self, percent):
        hashes_total = 66

        if percent <= self.last_percent:
            return

        hashes_needed = int (hashes_total * (float (percent) / 100))
        
        if sys.stdout.isatty ():
            sys.stdout.write ('    ')
                
        for i in range (hashes_needed):
            sys.stdout.write ('#')

        for i in range (hashes_needed, hashes_total):
            sys.stdout.write (' ')

        print "(%3d%%)" % percent,
        
        if (hashes_needed != hashes_total):
            for i in range (4 + hashes_total + 6):
                sys.stdout.write ("\b")
        else:
            sys.stdout.write ("\n")

        self.last_percent   = percent
        self.hashes_printed = hashes_needed

        sys.stdout.flush ()

def snapshot_progress (snapshooter, percent, progress):
    progress.update (percent)

def snapshot_finished (snapshooter):
    gtk.main_quit ()

def Usage ():
    print "sml-snapshooter: - Stateless Linux snapshooter tool\n\nUsage: stateless-snapshooter [-n|-l] [-p protosystem]"

if __name__ == '__main__':
    import sys
    import getopt

    if os.getuid () != 0:
        print "Please restart %s with root permissions!" % (sys.argv[0])
        sys.exit (1)

    if len (sys.argv) == 1:
        Usage ()
        sys.exit (1)

    do_new = False
    do_list = False
    protosystem = None

    class BadUsage: pass
    try:
        opts, args = getopt.getopt (sys.argv[1:], "nlp:h", ["new", "list", "protosystem", "help"])
        for opt, val in opts:
            if opt == '-n' or opt == '--new':
                do_new = True

            if opt == '-l' or opt == '--list':
                do_list = True

            if opt == '-p' or opt == '--protosytem':
                protosystem = val

            if opt == '-h' or opt == '--help':
                Usage ()
                sys.exit (0)

    except (getopt.error, BadUsage):
        Usage ()
        sys.exit (1)

    if (do_new and do_list):
        Usage ()
        sys.exit (1)

    if do_list:
        if protosystem != None:
            snapshots = snapshooter.list_snapshots (protosystem)
            if len (snapshots) > 0:
                print "Snapshots of %s: " % protosystem
                snapshots.sort () # FIXME: it'd be nice to sort numerically
                for snapshot in snapshots:
                    print "\t%s" % snapshot
            else:
                print "No snapshots of %s" % protosystem
        else:
            protosystems = snapshooter.list_protosystems ()
            if len (protosystems) > 0:
                print "Protosystems:"
                protosystems.sort ()
                for protosystem in protosystems:
                    print "\t%s" % protosystem
            else:
                print "No protosystems"
        
        sys.exit (0)

    if do_new:
        if protosystem == None:
            print "No protosystem specified"
            Usage ()
            sys.exit (0)

        progress = progressBar ()

        shooter = snapshooter.Snapshooter (protosystem,
                                           snapshooter.get_next_id (protosystem))

        shooter.get_transfer ().connect ('update', snapshot_progress, progress)
        shooter.connect ('snapshot-finished', snapshot_finished)
        shooter.start ()
        gtk.main ()
        sys.exit (0)
