#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import LDAPUtils
from config import *

def mac_address_to_filename (mac_address):
    # The "01" is the ARP hardware type for Ethernet which
    # pxelinux prepends to the MAC address before looking
    # up the configuration
    return "01-" + mac_address.lower ().replace (":", "-")

def write_client_cfg (mac_address, config_name, snapshot_name, server, snapshot_path):
    path = os.path.normpath ('/'.join ([PXEBASE, mac_address_to_filename (mac_address)]))
    f = file (path, 'w')
    f.write ("#\n# Generated by stateless-genpxecfg; Do not edit\n#\n\n")
    f.write ("default %s\n\n" % (config_name))
    f.write ("label %s\n" % (config_name))
    f.write ("  kernel stateless/%s/%s/vmlinuz\n" % (config_name, snapshot_name))
    f.write ("  append initrd=stateless/%s/%s/initrd.img NFSROOT=%s:%s\n" % 
             (config_name, snapshot_name, server, snapshot_path))
    f.close ()
    os.chmod (path, 0644)
    print "Wrote pxelinux config for %s to %s" % (mac_address, path)

def write_global_cfg (configurations):
    path = os.path.normpath ('/'.join ([PXEBASE, "default"]))
    f = file (path, 'w')
    if len (configurations) > 0:
        f.write ("#\n# Generated by stateless-genpxecfg; Do not edit\n#\n\n")
        f.write ("default %s\n\n" % (configurations[0][0]))
        for (config_name, snapshot_name, server, snapshot_path) in configurations:
            f.write ("label %s\n" % (config_name))
            f.write ("  kernel stateless/%s/%s/vmlinuz\n" % (config_name, snapshot_name))
            f.write ("  append initrd=stateless/%s/%s/initrd.img NFSROOT=%s:%s\n\n" % 
                     (config_name, snapshot_name, server, snapshot_path))
    f.close ()
    os.chmod (path, 0644)
    print "Wrote default pxelinux config to %s" % (path)

def generate_pxelinux_config ():
    cfg = StatelessConfig ()
    directory = LDAPUtils.ReadOnlyDirectory (cfg)
    srv = LDAPUtils.StatelessService (directory)

    for client in srv.get_client_list ():
        mac_address = client.get_attr_value ("macAddress")

        provisioning = srv.get_client_provisioning (mac_address)
        if provisioning == None or not provisioning.is_valid():
            print "Unable to get provisioning information for %s" % (mac_address)
            continue

        (server_name, snapshots_path) = srv.get_snapshot_location (provisioning.config_name,
                                                                   provisioning.snapshot_name,
                                                                   "nfs")
        if server_name == None or snapshots_path == None:
            print "Unable to get NFS location of %s snapshot %s" % \
                  (provisioning.config_name, provisioning.snapshot_name)
            continue

        snapshot_path = os.path.normpath ('/'.join ([snapshots_path,
                                                     provisioning.config_name,
                                                     provisioning.snapshot_name]))

        write_client_cfg (mac_address,
                          provisioning.config_name,
                          provisioning.snapshot_name,
                          server_name,
                          snapshot_path)

        configurations = []
        for cfg in srv.get_configuration_list ():
            config_name   = cfg.get_attr_value ("statelessConfigurationName")
            snapshot_name = cfg.get_attr_value ("statelessDefaultSnapshotName")

        assert config_name != None

        if snapshot_name == None:
            print "No default snapshot defined for %s" % (config_name)
            continue

        (server_name, snapshots_path) = srv.get_snapshot_location (config_name, snapshot_name, "nfs")
        if server_name == None or snapshots_path == None:
            print "Unable to get NFS location of %s snapshot %s" % (config_name, snapshot_name)
            continue

        snapshot_path = os.path.normpath ('/'.join ([snapshots_path,
                                                     provisioning.config_name,
                                                     provisioning.snapshot_name]))
        configurations.append ((config_name, snapshot_name, server_name, snapshot_path))

        write_global_cfg (configurations)

if __name__ == '__main__':
    import sys
    
    if os.getuid () != 0:
        print "Please restart %s with root permissions!" % (sys.argv[0])
        sys.exit (1)
                        
    generate_pxelinux_config ()
